/* Local Cyclades configuration and header file. See RELNOTES for more
   information and copyright */
   
/* User Settable Parameters
   The following values can go in this file, or in the kernel config
   file as "option" lines. In that case, remove the definitions here,
   or #ifndef them. If you do place them as option lines, make sure
   they read like:
	options		"CZ_DRIVER_VERSION=0x108"

*/

/* Driver Version Number */
#ifndef CZ_DRIVER_VERSION
#define CZ_DRIVER_VERSION 0x116
#endif

#ifndef CZPOLLVAL
#define CZPOLLVAL 25	/* If polling, poll this many times per second (rounded) */
#endif

/* Define our major number */
#ifndef CDEV_MAJOR
#define CDEV_MAJOR 48
#endif

/*
 *
 *CHANGE NOTHING BEYOND THIS POINT
 *
 *
 */

#define CZ_DTR_TICKS hz * 4 /* How many "ticks" to wait until reseting DTR off
				flag (4 second drop. Larger than the 3
				demanded by most modems) */


#ifndef CS_ODEVREADY
#define CS_ODEVREADY 0x20
#endif

#define MAX_BOARDS 4
#define MAX_INPUT_SIZE 1000

#define	MIN(a,b) (((a)<(b))?(a):(b))
#define	MAX(a,b) (((a)>(b))?(a):(b))

struct port_specific
  {
    struct CH_CTRL *channel_ctrl;
    struct BUF_CTRL *buffer_ctrl;
    long long flags;
    struct proc *p;
    struct tty ttyinfo;
    struct termios	it_in,
			it_out,
			lt_in,
			lt_out;
    int openers;
    unsigned char hotchar;
    int dtrwait;
  };
  
struct board_info
  {
    unsigned short	number_of_ports;
    pcici_t		config_id;
    char		*fpga;
    char		*window;
    struct RUNTIME_9060	*plx9060;
    struct BOARD_CTRL	*board_ctrl;
    struct FIRM_ID	*firmid;
    struct CUSTOM_REG	*custom_regs;
    struct ZFW_CTRL	*zfw_ctrl;
    struct port_specific ports[MAX_CHAN];
  };

struct czsoftc
  {
    struct board_info board[NCZ];
    unsigned long int	active_boards;
  } czcontrol;


#define CZ_PCI_PLX9060_MEM	0x10
#define CZ_PCI_PLX9060_PORT	0x14
#define CZ_PCI_MEMORY_WINDOW	0x18
#define CZ_PCI_IRQ		0x3C
#define CZ_PCI_FPGA_ADDRESS	0x40

#define loc_addr_base_offset	0x04
#define init_ctrl_offset	0x6C

#define RAM_SIZE		0x80000
#define PORT_SIZE		0x100


/* These are the macros to get the port information */
#define CZBOARD(czminor)	((czminor & 0xfff80000) >> 19)
#define CZPORT(czminor)		(czminor & 0xff)
#define CZPORTTYPE(czminor)	((czminor & 0x00070000) >> 16)

/* Board information */
#define PLX9060BUSY(board) czcontrol.board[board].plx9060->pci_doorbell != 0

/* Buffer Information */
#define RX_CHARS_IN_BUF(buf_ctrl) ((buf_ctrl->rx_put - buf_ctrl->rx_get + buf_ctrl->rx_bufsize) % buf_ctrl->rx_bufsize)
#define RX_SPACE_IN_BUF(buf_ctrl) ((buf_ctrl->rx_get - buf_ctrl->rx_put + buf_ctrl->rx_bufsize - 1) % buf_ctrl->rx_bufsize)
#define TX_CHARS_IN_BUF(buf_ctrl) ((buf_ctrl->tx_put - buf_ctrl->tx_get + buf_ctrl->tx_bufsize) % buf_ctrl->tx_bufsize)
#define TX_SPACE_IN_BUF(buf_ctrl) ((buf_ctrl->tx_get - buf_ctrl->tx_put + buf_ctrl->tx_bufsize - 1) % buf_ctrl->tx_bufsize)

/* Port field flags */
#define PORT_CALL_OUT_ACTIVE	1
#define PORT_SH_LOCK_ACTIVE	2
#define LOCK_STATE_SET	        4
#define INITIAL_STATE_SET	8
#define DTR_FORCED_OFF		16
#define IOCTL_PENDING		32
#define DO_POLL_WAKEUP		64

#define CALL_ACTIVE_MASK	3
#define LOCK_ACTIVE_MASK	12

/* Minor port types */
#define CALLIN_PORT		0
#define CALLOUT_PORT		1
#define CALLIN_INITIAL		2
#define CALLOUT_INITIAL		3
#define CALLIN_LOCK		4
#define CALLOUT_LOCK		5

/* CH_CTRL structures that can be altered (offsets) */
#define OP_MODE			0
#define INTR_ENABLE		1
#define RS_FLOW			2
#define FLOW_STATUS		3
#define COMM_BAUD		4
#define COMM_PARITY		5
#define COMM_DATA_L		6
#define COMM_FLAGS		7
#define RS_CONTROL		8
#define RS_STATUS		9
#define FLOW_XON		10
#define FLOW_XOFF		11

/* BUF_CTRL structures that can be altered (offsets) */

#define FLAG_DMA		0
#define TX_BUFADDR		1
#define TX_BUFSIZE		2
#define TX_THRESHOLD		3
#define TX_GET			4
#define TX_PUT			5
#define RX_BUFADDR		6
#define RX_BUFSIZE		7
#define RX_THRESHOLD		8
#define RX_GET			9
#define RX_PUT			10

/* BOARD_CTRL structures that can be altered (offsets) */
#define N_CHANNEL		0
#define FW_VERSION		1
#define OP_SYSTEM		2
#define DR_VERSION		3
#define INACTIVITY		4
#define HCMD_CHANNNEL		5
#define HCMD_PARAM		6
#define FWCMD_CHANNEL		7
#define FWCMD_PARAM		8


void set_initial_port_info(struct CH_CTRL *chctrl);

void cz_set_termios_params(unsigned int board, unsigned int port);
void cz_start(struct tty *tty);
int cz_param(struct tty *tty, struct termios *termios);
static void cz_dtr_on(void *devt);
static void disc_optim(struct tty *tp,struct termios *t, struct port_specific *com);
unsigned long int tx_moveable(unsigned long int put, unsigned long int get, unsigned long int size);
unsigned long int rx_moveable(unsigned long int put, unsigned long int get, unsigned long int size);


/* Stuff that should be in cyclom.h for the new boards, but isnt */

#define ZL_MAX_BLOCKS (16)

struct ZFILE_HEADER
  {
    char name[64];
    char date[32];
    char aux[32];
    unsigned long int n_config;
    unsigned long int config_offset;
    unsigned long int n_blocks;
    unsigned long int block_offset;
    unsigned long int reserved[9];
  };
struct ZFILE_CONFIG
  {
    char name[64];
    unsigned long int mailbox;
    unsigned long int function;
    unsigned long int n_blocks;
    unsigned long int block_list[ZL_MAX_BLOCKS];
  };

struct ZFILE_BLOCK
  {
    unsigned long int type;
    unsigned long int file_offset;
    unsigned long int ram_offset;
    unsigned long int size;
  };

#define ZFIRM_HLT 0x59505B5CL

unsigned char board_name[] = {
	'A','B','C','D','E','F','G','H','I','J','K','L','M',
	'N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
	'a','b','c','d','e','f','g','h','i','j','k','l','m',
	'n','o','p','q','r','s','t','u','v','w','x','y','z'};
